/* can_structures.h
 *
 * Command structure definitions
 *
 * This file can be used by Lua FFI.
 *
 */

#pragma pack(push)
#pragma pack(1)

enum { BIN_SAMPLE_DOMAIN = 0x00, DEC_SAMPLE_DOMAIN = 0x80};

typedef enum { CAN_TS_OFFSET = 1, CAN_GET_SYNC_TS, CAN_GET_TS,
               CAN_SETUP_SYNC_TS, CAN_PASSTHRU_POWER_ON, CAN_PASSTHRU_POWER_OFF,
               CAN_ENABLE_TERMINATOR, CAN_DISABLE_TERMINATOR,
               CAN_SETUP, CAN_RX_START, CAN_RX_STOP,
               CAN_TX_START, CAN_TX_MSG, CAN_TX_STOP,
               CAN_EVENT_SETUP, CAN_EVENT_START, CAN_EVENT_STOP,
               CAN_ENABLE_SELF_RX, CAN_DISABLE_SELF_RX,
               CAN_ADD_QUERY_MSG, CAN_CLEAR_QUERY_MSG,
               CAN_LAST_COMMAND = 0x7FFFFFFF    /* Insures 32-bit enum */
} can_cmd_t;

typedef enum { QUERY_BIN_0p1HZ = 0, QUERY_BIN_0p2HZ = 1, QUERY_BIN_0p4HZ = 2,
               QUERY_BIN_0p8HZ = 3, QUERY_BIN_1p6HZ = 4, QUERY_BIN_3p2HZ = 5,
               QUERY_BIN_6p4HZ = 6, QUERY_BIN_12p8HZ = 7, QUERY_BIN_25p6HZ = 8,
               QUERY_BIN_51p2HZ = 9, QUERY_BIN_102p4HZ = 10,
               QUERY_BIN_128HZ = 11, QUERY_BIN_204p8HZ = 12,
               QUERY_DEC_0p1HZ = (DEC_SAMPLE_DOMAIN + 0),
               QUERY_DEC_0p2HZ = (DEC_SAMPLE_DOMAIN + 1),
               QUERY_DEC_0p5HZ = (DEC_SAMPLE_DOMAIN + 2),
               QUERY_DEC_1HZ = (DEC_SAMPLE_DOMAIN + 3),
               QUERY_DEC_2HZ = (DEC_SAMPLE_DOMAIN + 4),
               QUERY_DEC_5HZ = (DEC_SAMPLE_DOMAIN + 5),
               QUERY_DEC_10HZ = (DEC_SAMPLE_DOMAIN + 6),
               QUERY_DEC_20HZ = (DEC_SAMPLE_DOMAIN + 7),
               QUERY_DEC_50HZ = (DEC_SAMPLE_DOMAIN + 8),
               QUERY_DEC_100HZ = (DEC_SAMPLE_DOMAIN + 9),
               QUERY_DEC_200HZ = (DEC_SAMPLE_DOMAIN + 10),
               QUERY_UNUSED_RATE = 0xFF
} can_query_rate_t;

typedef struct {
    uint8_t                     sjw;                /* 1 to 4 */
    uint8_t                     brp;                /* 1 to 64 */
    uint8_t                     tseg1;              /* 1 to 16 */
    uint8_t                     tseg2;              /* 1 to 8 */
    uint8_t                     samples_per_bit;    /* 1 or 3 */
    uint8_t                     listen_only;        /* 0 or 1 */
    uint8_t                     reserved[2];
} can_cmd_setup_s;

typedef struct {
    int64_t                     start_time;
    cmd_rate_parameters_s       heartbeat_interval;
    uint32_t                    phys_address;
} can_cmd_rx_start_s;

typedef struct {
    uint32_t                    phys_address;
    uint32_t                    channel_id;
} can_cmd_tx_start_stop_s;

typedef struct {
    uint32_t        phys_address;
} can_cmd_add_query_msg_s;

typedef struct {
    uint8_t         bus_off_deassert;
    uint8_t         bus_off;
    uint8_t         reserved[2];
} can_cmd_event_vector_s;

typedef struct {
    uint32_t          can_id;
    can_query_rate_t  sample_rate;
    uint8_t           can_message[8];
    uint8_t           data_length;
} can_query_msg_s;

typedef struct {
    int64_t     ts;
    uint32_t    can_id;
    uint8_t     can_message[8];
    uint8_t     frame_format;
    uint8_t     tx_error_count;
    uint8_t     rx_error_count;
    uint8_t     error_capture_code;
} can_rxtx_buffer_t;

typedef struct {
    can_cmd_t       cmd;
    uint32_t        cmd_id;
    union {
        cmd_ts_offset_s         ts_offset;
        can_cmd_setup_s         can_setup;
        can_cmd_rx_start_s      rx_start;
        can_cmd_tx_start_stop_s tx_start_stop;
        can_cmd_add_query_msg_s add_query_msg;
        can_rxtx_buffer_t       tx_msg;
        cmd_event_start_s       ev_start;
        can_cmd_event_vector_s  events;
    } prm;
} can_cmd_s;

typedef struct {
    int64_t                     ts;
    can_cmd_event_vector_s      events;
} can_ev_buffer_t;

enum { MAX_CAN_TX_STARTS = 16 };

enum { MAX_NUM_SAMPLE_RATES = 16 };
enum { NUM_ACTIVE_QUERIES = 75};
enum { QUERY_OK = 0, BAD_SAMPLE_DOMAIN = 1, OUT_OF_BUFFERS = 2 };
enum { SAMPLE_DOMAIN_MASK = 0x80 };

#pragma pack(pop)
