--
-- Vivotek
--
-- Method: GET (Video)
-- Syntax:
--     http://<camera_ip>/video.mjpg
--
-- Method: GET (Parameters)
-- Syntax:
--     http://<camera_ip>/cgi-bin/admin/{get|set}param.cgi
--             [?<parameter>=<value>[&<parameter>=<value>...]]
--
-- Parameter Pairs
-- ---------------------------------------------------------------------
-- [ system_info_modelname ]
-- [ system_info_serialnumber ]
-- [ system_info_firmwareversion ]
--
-- FIXME: The remaining CGI API from documentation

local __ = require '__'
local log = require 'log'
local class = require 'class'
local camera = require 'camera/common'
local ffi = require 'ffi'
local C = ffi.C

-- GNU C Library 'libc'

ffi.cdef[[
int sscanf(const char *str, const char *format, ...);
char *strstr(const char *haystack, const char *needle);
char *strerror(int);
]]

local m = class(camera)

function m:setup(vendor, model, serial, host, port)
    camera:setup(vendor, model, serial, host, port)
    local query = ''
    if ffi.string(self.camera[0].query) ~= query then
        ffi.copy(self.camera[0].query, query)
    end
    ffi.copy(self.camera[0].page, 'video.mjpg')
    -- set Vivotek camera parameters (no onliner for Vivotek)
    local line = ffi.new('char *')
    local result = ffi.new('char [81]')
    local rbuf = ffi.new('char [512]')
    local ip = ffi.string(self.camera[0].host)
    local image_size = ffi.string(self.camera[0].spec_image_size)
    --TODO: Vivotek uses quality values and not compression
    local image_comp = ffi.string(self.camera[0].spec_image_compression)
    local qv = math.abs(tonumber(image_comp) - 100)
    if qv < 5 then
        qv = 5
    elseif qv > 95 then
        qv = 95
    end
    local qvalue = tostring(qv)
    local frame_rate = ffi.string(self.camera[0].spec_frame_rate)
    query = "POST /cgi-bin/admin/setparam.cgi HTTP/1.0\r\n" ..
            "Host: " .. ip .. "\r\n" ..
            "User-Agent: HBM CX23R\r\n" ..
            "Content-Type: application/x-www-form-urlencoded\r\n" ..
            "Content-Length: 29\r\n\r\n" ..
            "videoin_c0_s0_codectype=mjpeg"
    ffi.fill(rbuf, ffi.sizeof(rbuf), 0)
    if self:info(ip, query, rbuf) then
        line = C.strstr(rbuf, "videoin_c0_s0_codectype")
        if line ~= nil then
            C.sscanf(line, "videoin_c0_s0_codectype='%5[^']", result)
            if ffi.string(result) ~= 'mjpeg' then
                io.stderr:write(ffi.string(self.camera[0].name) ..
                                ":setup(): codec " ..
                                ffi.string(result) .. " ~= mjpeg\n")
            end
        end
    end
    query = "POST /cgi-bin/admin/setparam.cgi HTTP/1.0\r\n" ..
            "Host: " .. ip .. "\r\n" ..
            "User-Agent: HBM CX23R\r\n" ..
            "Content-Type: application/x-www-form-urlencoded\r\n" ..
            "Content-Length: 27\r\n\r\n" ..
            "videoin_c0_s0_mjpeg_quant=3"
    ffi.fill(rbuf, ffi.sizeof(rbuf), 0)
    if self:info(ip, query, rbuf) then
        line = C.strstr(rbuf, "videoin_c0_s0_mjpeg_quant")
        if line ~= nil then
            C.sscanf(line, "videoin_c0_s0_mjpeg_quant='%3[^']", result)
            if ffi.string(result) ~= '3' then
                io.stderr:write(ffi.string(self.camera[0].name) ..
                                ":setup(): mjpeg_quant " ..
                                ffi.string(result) .. " ~= 3\n")
            end
        end
    end
    query = "POST /cgi-bin/admin/setparam.cgi HTTP/1.0\r\n" ..
            "Host: " .. ip .. "\r\n" ..
            "User-Agent: HBM CX23R\r\n" ..
            "Content-Type: application/x-www-form-urlencoded\r\n" ..
            "Content-Length: " .. tostring(#image_size + 25) .. "\r\n\r\n" ..
            "videoin_c0_s0_resolution=" .. image_size
    ffi.fill(rbuf, ffi.sizeof(rbuf), 0)
    if self:info(ip, query, rbuf) then
        line = C.strstr(rbuf, "videoin_c0_s0_resolution")
        if line ~= nil then
            C.sscanf(line, "videoin_c0_s0_resolution='%10[^']", result)
            if ffi.string(result) ~= image_size then
                io.stderr:write(ffi.string(self.camera[0].name) ..
                                ":setup(): resolution " ..
                                ffi.string(result) .. " ~= " ..
                                image_size .. "\n")
            end
        end
    end
    query = "POST /cgi-bin/admin/setparam.cgi HTTP/1.0\r\n" ..
            "Host: " .. ip .. "\r\n" ..
            "User-Agent: HBM CX23R\r\n" ..
            "Content-Type: application/x-www-form-urlencoded\r\n" ..
            "Content-Length: " .. tostring(#qvalue + 27) .. "\r\n\r\n" ..
            "videoin_c0_s0_mjpeg_qvalue=" .. qvalue
    ffi.fill(rbuf, ffi.sizeof(rbuf), 0)
    if self:info(ip, query, rbuf) then
        line = C.strstr(rbuf, "videoin_c0_s0_mjpeg_qvalue")
        if line ~= nil then
            C.sscanf(line, "videoin_c0_s0_mjpeg_qvalue='%3[^']", result)
            if ffi.string(result) ~= qvalue then
                io.stderr:write(ffi.string(self.camera[0].name) ..
                                ":setup(): compression " ..
                                image_comp .. " is INVALID\n")
            end
        end
    end
    query = "POST /cgi-bin/admin/setparam.cgi HTTP/1.0\r\n" ..
            "Host: " .. ip .. "\r\n" ..
            "User-Agent: HBM CX23R\r\n" ..
            "Content-Type: application/x-www-form-urlencoded\r\n" ..
            "Content-Length: " .. tostring(#frame_rate + 29) .. "\r\n\r\n" ..
            "videoin_c0_s0_mjpeg_maxframe=" .. frame_rate
    ffi.fill(rbuf, ffi.sizeof(rbuf), 0)
    if self:info(ip, query, rbuf) then
        line = C.strstr(rbuf, "videoin_c0_s0_mjpeg_maxframe")
        if line ~= nil then
            C.sscanf(line, "videoin_c0_s0_mjpeg_maxframe='%2[^']", result)
            if ffi.string(result) ~= frame_rate then
                io.stderr:write(ffi.string(self.camera[0].name) ..
                                ":setup(): frame_rate " ..
                                ffi.string(result) .. " ~= " ..
                                frame_rate .. "\n")
            end
        end
    end
    return self
end

function m:discover(arp)
    local o = {}
    for i = 1, #arp do
        if arp[i].serial:match('0002d1') then
            table.insert(o, { vendor = 'vivotek', ip = arp[i].ip })
        end
    end
    if #o == 0 then
        -- No Vivotek serial numbers found
        return o
    end
    local list = {}
    local query = {}
    local line = ffi.new('char *')
    local _model = ffi.new('char [33]')
    local _serial = ffi.new('char [33]')
    local _firmware = ffi.new('char [33]')
    local rbuf = ffi.new('char [512]')
    for i = 1, #o do
        query = "POST /cgi-bin/admin/getparam.cgi HTTP/1.0\r\n" ..
                "Host: " .. o[i].ip .. "\r\n" ..
                "User-Agent: HBM CX23R\r\n" ..
                "Content-Type: application/x-www-form-urlencoded\r\n" ..
                "Content-Length: 21\r\n\r\n" ..
                "system_info_modelname"
        ffi.fill(rbuf, ffi.sizeof(rbuf), 0)
        if self:info(o[i].ip, query, rbuf) then
            line = C.strstr(rbuf, "system_info_modelname")
            if line ~= nil then
                ffi.fill(_model, ffi.sizeof(_model), 0)
                C.sscanf(line, "system_info_modelname='%32[^']", _model)
                o[i].model = ffi.string(_model)
            end
        else
            err = string.format("discover(): could not query %s camera @ %s",
                                o[i].vender, o[i].ip)
            io.stdout:write(err)
            goto continue
        end
        query = "POST /cgi-bin/admin/getparam.cgi HTTP/1.0\r\n" ..
                "Host: " .. o[i].ip .. "\r\n" ..
                "User-Agent: HBM CX23R\r\n" ..
                "Content-Type: application/x-www-form-urlencoded\r\n" ..
                "Content-Length: 24\r\n\r\n" ..
                "system_info_serialnumber"
        ffi.fill(rbuf, ffi.sizeof(rbuf), 0)
        if self:info(o[i].ip, query, rbuf) then
            line = C.strstr(rbuf, "system_info_serialnumber")
            if line ~= nil then
                ffi.fill(_serial, ffi.sizeof(_serial), 0)
                C.sscanf(line, "system_info_serialnumber='%32[^']", _serial)
                o[i].serial = string.lower(ffi.string(_serial))
            end
        else
            err = string.format("discover(): could not query %s camera @ %s",
                                o[i].vender, o[i].ip)
            io.stdout:write(err)
            goto continue
        end
        query = "POST /cgi-bin/admin/getparam.cgi HTTP/1.0\r\n" ..
                "Host: " .. o[i].ip .. "\r\n" ..
                "User-Agent: HBM CX23R\r\n" ..
                "Content-Type: application/x-www-form-urlencoded\r\n" ..
                "Content-Length: 27\r\n\r\n" ..
                "system_info_firmwareversion"
        ffi.fill(rbuf, ffi.sizeof(rbuf), 0)
        if self:info(o[i].ip, query, rbuf) then
            line = C.strstr(rbuf, "system_info_firmwareversion")
            if line ~= nil then
                ffi.fill(_firmware, ffi.sizeof(_firmware), 0)
                C.sscanf(line, "system_info_firmwareversion='%32[^']", _firmware)
                o[i].firmware = ffi.string(_firmware)
            end
        else
            err = string.format("discover(): could not query %s camera @ %s",
                                o[i].vender, o[i].ip)
            io.stdout:write(err)
            goto continue
        end
        if o[i].model and o[i].serial and o[i].firmware then
            table.insert(list, o[i])
        else
            if not o[i].model then
                err = string.format("discover: no model from %s camera @ %s",
                           o[i].vender, o[i].ip)
                io.stderr:write(err)
            end
            if not o[i].serial then
                err = string.format("discover: no serial from %s camera @ %s",
                           o[i].vender, o[i].ip)
                io.stderr:write(err)
            end
            if not o[i].firmware then
                err = string.format("discover: no firmware from %s camera @ %s",
                           o[i].vender, o[i].ip)
                io.stderr:write(err)
            end
        end
        ::continue::
    end
    return list
end

function m:snapshot(proxy, file)
    if not file then
        log:error(__"{0}_camera:snapshot missing filename on {1}",
                  self:vendor(), proxy.proxy_hostname)
    end
    local fn = '/tmp/' .. file
    self.fp = C.fopen(fn, 'wb')
    if self.fp ~= nil then
        os.execute('curl -s -k http://' .. self:host() ..
                   '/cgi-bin/viewer/video.jpg' ..
                   ' -o ' .. fn)
        os.execute('ln -sf ' .. fn ..
                   ' /usr/share/web/static/images/' .. file)
    else
        log:error(__"{0}_camera:snapshot {1} on {2}", self:vendor(),
                  ffi.string(C.strerror(ffi.errno())), proxy.proxy_hostname)
    end
    return self.fp
end

return m
