local ffi = require "ffi"

local libsie = ffi.load("libsie.so.0")

ffi.cdef[[

enum {
    SIE_OUTPUT_NONE = 0,
    SIE_OUTPUT_FLOAT64 = 1,
    SIE_OUTPUT_RAW = 2
};

typedef double sie_float64;
typedef unsigned int sie_uint32;

typedef void sie_Context;
typedef void sie_File;
typedef void sie_Iterator;
typedef void sie_Test;
typedef void sie_Tag;
typedef void sie_Spigot;
typedef void sie_Output;
typedef void sie_Channel;
typedef void sie_Dimension;
typedef void sie_Histogram;
typedef void sie_Exception;
typedef void sie_Stream;

typedef struct _sie_Output_Raw {
    void *ptr;
    size_t size;
    int reserved_1;
} sie_Output_Raw;

typedef struct _sie_Output_Dim {
    int type;
    sie_float64 *float64;
    sie_Output_Raw *raw;
} sie_Output_Dim;

typedef struct _sie_Output_Struct {
    size_t num_dims;
    size_t num_rows;
    size_t reserved_1;
    size_t reserved_2;
    sie_Output_Dim *dim;
} sie_Output_Struct;

typedef int (sie_Progress_Set_Message)(void *data, const char *message);
typedef int (sie_Progress_Percent)(void *data, sie_float64 percent_done);

sie_Context *sie_context_new(void);
void *sie_retain(void *object);
void sie_release(void *object);
int sie_context_done(sie_Context *context);
void sie_free(void *pointer);
sie_File *sie_file_open(void *context_object, const char *name);
sie_Iterator *sie_get_tests(void *reference);
sie_Iterator *sie_get_channels(void *reference);
sie_Iterator *sie_get_dimensions(void *reference);
sie_Iterator *sie_get_tags(void *reference);
sie_Test *sie_get_test(void *reference, sie_uint32 id);
sie_Channel *sie_get_channel(void *reference, sie_uint32 id);
sie_Dimension *sie_get_dimension(void *reference, sie_uint32 index);
sie_Tag *sie_get_tag(void *reference, const char *id);
sie_Test *sie_get_containing_test(void *reference);
const char *sie_get_name(void *reference);
sie_uint32 sie_get_id(void *reference);
sie_uint32 sie_get_index(void *reference);
void *sie_iterator_next(void *iter);
const char *sie_tag_get_id(sie_Tag *tag);
char *sie_tag_get_value(sie_Tag *tag);
int sie_tag_get_value_b(sie_Tag *tag, const char **value, size_t *size);
sie_Spigot *sie_attach_spigot(void *reference);
sie_Output *sie_spigot_get(sie_Spigot *spigot);
void sie_spigot_disable_transforms(void *spigot, int disable);
void sie_spigot_transform_output(void *spigot, sie_Output *output);
size_t sie_spigot_seek(void *spigot, size_t target);
size_t sie_spigot_tell(void *spigot);
int sie_lower_bound(void *spigot, size_t dim, sie_float64 value,
    size_t *block, size_t *scan);
int sie_upper_bound(void *spigot, size_t dim, sie_float64 value,
    size_t *block, size_t *scan);
size_t sie_output_get_block(sie_Output *output);
size_t sie_output_get_num_dims(sie_Output *output);
size_t sie_output_get_num_rows(sie_Output *output);
int sie_output_get_type(sie_Output *output, size_t dim);
sie_float64 *sie_output_get_float64(sie_Output *output, size_t dim);
sie_Output_Raw *sie_output_get_raw(sie_Output *output, size_t dim);
sie_Output_Struct *sie_output_get_struct(sie_Output *output);
sie_Exception *sie_check_exception(void *ctx_obj);
sie_Exception *sie_get_exception(void *ctx_obj);
char *sie_report(void *exception);
char *sie_verbose_report(void *exception);
void sie_set_progress_callbacks(void *ctx_obj, void *data, sie_Progress_Set_Message *set_message_callback, sie_Progress_Percent *percent_callback);
sie_Stream *sie_stream_new(void *context_object);
size_t sie_add_stream_data(void *stream, const void *data, size_t size);
int sie_spigot_done(void *spigot);
int sie_file_is_sie(void *ctx_obj, const char *name);
void sie_ignore_trailing_garbage(void *ctx_obj, size_t amount);
sie_Histogram *sie_histogram_new(sie_Channel *channel);
size_t sie_histogram_get_num_dims(sie_Histogram *hist);
size_t sie_histogram_get_num_bins(sie_Histogram *hist, size_t dim);
void sie_histogram_get_bin_bounds(sie_Histogram *hist, size_t dim,
    sie_float64 *lower, sie_float64 *upper);
sie_float64 sie_histogram_get_bin(sie_Histogram *hist, size_t *indices);
sie_float64 sie_histogram_get_next_nonzero_bin(sie_Histogram *hist,
    size_t *start, size_t *indices);
void sie_system_free(void *pointer);
int sie_binary_search(void *spigot, size_t dim, sie_float64 value,
    size_t *block, size_t *scan);

/* sie writer

typedef uint32_t sie_id;
typedef size_t (sie_Writer_Fn)(void *user, const char *data, size_t size);

typedef enum _sie_Writer_Id_Type {
    SIE_WRITER_ID_GROUP,
    SIE_WRITER_ID_TEST,
    SIE_WRITER_ID_CH,
    SIE_WRITER_ID_DECODER,
    SIE_WRITER_ID_NUM_TYPES
} sie_Writer_Map_Type;

struct _sie_Context_Object {
    sie_Object parent;
    sie_Context *context;
};

typedef struct _sie_Context_Object sie_Context_Object;

struct _sie_Writer {
    sie_Context_Object parent;
    sie_Writer_Fn *writer_fn;
    void *user;
    char *xml_buf;
    char *index_buf;
    sie_uint64 offset;
    int do_index;
    sie_id next_ids[SIE_WRITER_ID_NUM_TYPES];
};

typedef struct _sie_Writer sie_Writer;

sie_Writer *sie_writer_new(void *ctx_obj, sie_Writer_Fn *writer_fn, void *user);
void sie_writer_init(sie_Writer *self, void *ctx_obj, sie_Writer_Fn *writer_fn,
    void *user);
void sie_writer_destroy(sie_Writer *self);
int sie_writer_write_block(sie_Writer *self, sie_id group, const char *data,
    size_t size);
int sie_writer_xml_string(sie_Writer *self, const char *data, size_t size);
int sie_writer_xml_node(sie_Writer *self, sie_XML *node);
int sie_writer_xml_header(sie_Writer *self);
int sie_writer_flush_xml(sie_Writer *self);
sie_id sie_writer_next_id(sie_Writer *self, int type);
void sie_writer_prepare_append(sie_Writer *self, void *intake);
sie_uint64 sie_writer_total_size(sie_Writer *self, sie_uint64 addl_bytes,
    sie_uint64 addl_blocks);
*/

]]

return libsie
