require 'strict'
local ffi = require 'ffi'
local config = require 'system_config'
local libte = ffi.load(config.test_engine_library)

ffi.cdef[[
    struct min_max_data {
        double minimum;
        double maximum;
    };
    int calculate_min_max_float32(const struct pipe *p,
        struct min_max_data *data);
    int calculate_min_max_float64(const struct pipe *p,
        struct min_max_data *data);
]]

if config.use_libte then
    return {
        calculate_min_max_float32 = libte.calculate_min_max_float32,
        calculate_min_max_float64 = libte.calculate_min_max_float64,
    }
else
    return {
        calculate_min_max_float32 = function() return -1 end,
        calculate_min_max_float64 = function() return -1 end,
    }
end
