require 'strict'

-- trigger.lua - Implements a "one shot" trigger for use by a data mode.
--
-- This kind of trigger turns on for one sample when the trigger value
-- sample is true with the previous value being false.  If the new
-- trigger value is false this trigger resets and can fire on the next
-- false-to-true transition.

local trig_common = require 'test_engine/modules/trigger/trig_common'
local mt = {}

-- Determines whether the given value has caused the trigger
-- to be on or off.
-- Parameter:
--     'trig_val'  is a sample value from a logical trigger channel
-- Returns:  'true' if trigger is on, false otherwise
function mt:triggered(trig_val)
    if trig_val ~= 0 and not self.prev_trigger then
        self.prev_trigger = true
        return true
    elseif trig_val == 0 then
        self.prev_trigger = false
    end
    return false
end

local m = {}

-- Creates a new trigger object.
-- Parameters:
--   'in_chans'  input channel names
--   'trig_chan' trigger channel name
--   'node_list' list of previously instantiated channel nodes
--   'stanza_list'  list of node metadata stanzas
-- Returns:
--   the new instance object
function m:new(in_chans, trig_chan, node_list, stanza_list)
    assert(in_chans, "missing first parameter")
    assert(trig_chan, "missing second parameter")
    assert(node_list, "missing third parameter")

    local trig_core = setmetatable({}, {__index = mt})
    trig_core.prev_trigger = false

    local trig_obj =
        trig_common:new(in_chans, trig_chan, node_list, trig_core, stanza_list)

    return trig_obj
end

return m
